# {{ ansible_managed }}
user {{ nginx_user }};
worker_processes 2;
pid /run/nginx.pid;

events {
	worker_connections 768;
	# multi_accept on;
}

http {

	##
	# Basic Settings
	##

	sendfile on;
	tcp_nopush on;
	tcp_nodelay on;
	keepalive_timeout 65;
	types_hash_max_size 2048;
	# server_tokens off;

	# server_names_hash_bucket_size 64;
	# server_name_in_redirect off;

	include /etc/nginx/mime.types;
	default_type application/octet-stream;

	##
	# Logging Settings
	##

	access_log /var/log/nginx/access.log;
	error_log /var/log/nginx/error.log;

	##
	# Gzip Settings
	##

	gzip on;

	server {
		listen {{ file_url_port }};
		server_name {{ ansible_hostname }};
		root {{ http_boot_folder }};
		location {{ http_boot_folder }}/ {
			alias {{ http_boot_folder }}/;
		}
	}
        include /etc/nginx/conf.d/bifrost*.conf;
}
